#include "SPI.h"
#include "ws2812.h"

#define NUM_LEDS  120
#define MAX_CH    (NUM_LEDS/2)

//Defines and variables for Radio
#define RFCHANNEL        0       // Let's use channel 0
#define SYNCWORD1        0xB5    // Synchronization word, high byte
#define SYNCWORD0        0x47    // Synchronization word, low byte
#define SOURCE_ADDR      5       // Sender address
#define DESTINATION_ADDR 4       // Receiver address

volatile uint8_t Llenght;
volatile uint8_t Rlenght;
volatile uint8_t packetAvailable=0;

WS2812STRIP strip(NUM_LEDS);

void fillAudio(uint8_t fLeft, uint8_t fRight) {
    int index;
    for (index=0; index < MAX_CH; index++) {
        if (fLeft > index) {
            if (index<=5) strip.setLEDColor((MAX_CH-index-1),fLeft/2, 0x00, fLeft/2);
            else
            if (index<=14) strip.setLEDColor((MAX_CH-index-1),0x00, 0x00, fLeft/2);
            else
            if (index<=23) strip.setLEDColor((MAX_CH-index-1),0x00, fLeft/2, 0x00);
            else strip.setLEDColor((MAX_CH-index-1),fLeft/2, 0x00, 0x00);
        } else strip.setLEDColor((MAX_CH-index-1),0x00, 0x00, 0x00);
        if (fRight >index) {
            if (index<=5) strip.setLEDColor((MAX_CH+index),fRight/2, 0x00, fRight/2);
            else
            if (index<=14) strip.setLEDColor((MAX_CH+index),0x00, 0x00, fRight/2);
            else
            if (index<=23) strip.setLEDColor((MAX_CH+index),0x00, fRight/2, 0x00);
            else strip.setLEDColor((MAX_CH+index),fRight/2, 0x00, 0x00);
        }else strip.setLEDColor((MAX_CH+index),0x00, 0x00, 0x00);
    }
}


void fillAudio2(uint8_t fLeft, uint8_t fRight) {
    uint8_t index;
    for (index=0; index < MAX_CH; index++) {
        if ((fLeft>0) && (fLeft >= index)) {
          strip.setLEDColor((MAX_CH-index),fLeft, (MAX_CH-fLeft), 0x00);
        } else strip.setLEDColor((MAX_CH-index-1),0x00, 0x00, 0x01);
        if ((fLeft>0) && (fLeft >= index)) {
          strip.setLEDColor((MAX_CH+index), fRight, (MAX_CH-fRight), 0x00);
        }else strip.setLEDColor((MAX_CH+index),0x00, 0x00, 0x01);
    }
}

void fillAudio3(uint8_t fLeft, uint8_t fRight) {
    uint8_t index;
    for (index=0; index < MAX_CH; index++) {
        if ((fLeft > 0) && (fLeft >= index) && (fLeft-10 <= index)) {
          strip.setLEDColor((MAX_CH-index),fLeft, (MAX_CH-fLeft), 0x00);
        } else strip.setLEDColor((MAX_CH-index-1),0x00, 0x00, 0x01);
        if ((fRight>0) && (fRight >= index) && (fRight-10 <= index)) {
          strip.setLEDColor((MAX_CH+index), fRight, (MAX_CH-fRight), 0x00);
        }else strip.setLEDColor((MAX_CH+index),0x00, 0x00, 0x01);
    }
}


void rfPacketReceived(CCPACKET *packet)
{   
  if (packet->length > 0) {
    Llenght = packet->data[1];
    Rlenght = packet->data[2];
    packetAvailable = packet->data[0];
  }
}

void setup()
{
    strip.begin();
    panstamp.radio.setChannel(RFCHANNEL);
    panstamp.radio.setSyncWord(SYNCWORD1, SYNCWORD0);
    panstamp.radio.setCarrierFreq(CFREQ_433);
    panstamp.radio.disableAddressCheck();
    panstamp.attachInterrupt(rfPacketReceived);
    panstamp.radio.setDevAddress(SOURCE_ADDR);
    panstamp.rxOn();
}

void loop()
{
  interrupts();
  while(packetAvailable==0);
  switch (packetAvailable) {
    case 1: { fillAudio(Llenght, Rlenght); break;}
    case 2: { fillAudio2(Llenght, Rlenght); break;}
    case 3: { fillAudio3(Llenght, Rlenght); break;}
  }
  packetAvailable=0;
  panstamp.rxOn();
  strip.show();
}
