#define SHOWMODE 1

#define EXP         1.4
#define SMOOTH      0.5
#define MAX_COEF    1.8
#define LOW_PASS    100
#define MAXLEVEL    5800
#define NUM_LEDS    120
#define MAX_CH      (NUM_LEDS/2)

//Defines and variables for Radio
#define RFCHANNEL        0       // Let's use channel 0
#define SYNCWORD1        0xB5    // Synchronization word, high byte
#define SYNCWORD0        0x47    // Synchronization word, low byte
#define SOURCE_ADDR      4       // Sender address
#define DESTINATION_ADDR 5       // Receiver address

CCPACKET txPacket;

extern uint16_t analogRef;
volatile uint16_t Lvalue = 0;
volatile uint16_t Rvalue = 0;
volatile float Ltmpvalue = 0;
volatile float Rtmpvalue = 0;
volatile float Lfvalue;
volatile float Rfvalue;
volatile uint8_t Llenght;
volatile uint8_t Rlenght;
unsigned long timing;


void ADCinit() {
  //Set input direction for A0 (Left audio) and A1 (Right audio)
  P2DIR &= ~(A0 + A1);
  //Select alternative (analog) functions, A5 - External VRef
  P2SEL |= (A0 + A1 + A5);
  //ADC-On, 64 ADC12CLK cycles, sample-and-conversions are performed automatically
  ADC12CTL0 = ADC12ON | ADC12SHT0_2 | ADC12MSC;
  //signal is sourced from the sampling timer, SMCLK, = Sequence-of-channels, clock divider 1
  ADC12CTL1 = ADC12SHP | ADC12SSEL_3 | ADC12CONSEQ_1 | ADC12DIV_1;
  //Input channel select A0, V(R+) = VeREF+ (external Ref)
  ADC12MCTL0 = ADC12INCH_0 | ADC12SREF_2;
  //Input channel select A1, V(R+) = VeREF+ (external Ref), = End of sequence
  ADC12MCTL1 = ADC12INCH_1 | ADC12SREF_2 | ADC12EOS;
  //Enable ADC
  ADC12CTL0 |= ADC12ENC;
}

void GetAudio() {
  int i;
  Ltmpvalue = 0;
  Rtmpvalue = 0;
  for (i = 0; i < 200;i++) {
    ADC12CTL0 |= ADC12SC;
    while (!(ADC12IFG & BIT0));
    while (!(ADC12IFG & BIT1));
    Lvalue=ADC12MEM0;
    Rvalue=ADC12MEM1;
    if ((float)Lvalue > Ltmpvalue) Ltmpvalue = (float)Lvalue;
    if ((float)Rvalue > Rtmpvalue) Rtmpvalue = (float)Rvalue;
  }
}
void setup()
{
    ADCinit();
    panstamp.radio.setChannel(RFCHANNEL);
    panstamp.radio.setSyncWord(SYNCWORD1, SYNCWORD0);
    panstamp.radio.setDevAddress(SOURCE_ADDR);
    panstamp.radio.disableAddressCheck();
    panstamp.setLowTxPower();
    panstamp.radio.setCarrierFreq(CFREQ_433);
}

void loop()
{
  if (millis() - timing > 20) {
    timing = millis();
    GetAudio();

    Ltmpvalue=map(Ltmpvalue,LOW_PASS,4096,0,500);
    Rtmpvalue=map(Rtmpvalue,LOW_PASS,4096,0,500);

    Ltmpvalue=constrain(Ltmpvalue,0,500);
    Rtmpvalue=constrain(Rtmpvalue,0,500);

    Ltmpvalue=powf(Ltmpvalue, EXP);
    Rtmpvalue=powf(Rtmpvalue, EXP);

    Lfvalue=Ltmpvalue*SMOOTH + Lfvalue*(1-SMOOTH);
    Rfvalue=Rtmpvalue*SMOOTH + Rfvalue*(1-SMOOTH);

    Llenght = map(Lfvalue, 0, MAXLEVEL, 0, MAX_CH);
    Rlenght = map(Rfvalue, 0, MAXLEVEL, 0, MAX_CH);
    Llenght = constrain(Llenght, 0, MAX_CH);
    Rlenght = constrain(Rlenght, 0, MAX_CH);

    txPacket.length = 3;
    txPacket.data[0] = SHOWMODE;      // Audio show mode
    txPacket.data[1] = Llenght;       // Left level
    txPacket.data[2] = Rlenght;       // Right level
    panstamp.radio.sendData(txPacket);// Transmit packet
  }
}
