
#include "SPI.h"
#include "ws2812.h"

#define EXP         1.4
#define SMOOTH      0.5
#define MAX_COEF    1.8
#define LOW_PASS    100
#define MAXLEVEL    5800
#define NUM_LEDS    120
#define MAX_CH      (NUM_LEDS/2)

extern uint16_t analogRef;
volatile uint16_t Lvalue = 0;
volatile uint16_t Rvalue = 0;
volatile float Ltmpvalue = 0;
volatile float Rtmpvalue = 0;
volatile float Lfvalue;
volatile float Rfvalue;
volatile uint8_t Llenght;
volatile uint8_t Rlenght;
unsigned long timing;

WS2812STRIP strip(NUM_LEDS);

void ADCinit() {
  //Set input direction for A0 (Left audio) and A1 (Right audio)
  P2DIR &= ~(A0 + A1);
  //Select alternative (analog) functions, A5 - External VRef
  P2SEL |= (A0 + A1 + A5);
  //ADC-On, 64 ADC12CLK cycles, sample-and-conversions are performed automatically
  ADC12CTL0 = ADC12ON | ADC12SHT0_2 | ADC12MSC;
  //signal is sourced from the sampling timer, SMCLK, = Sequence-of-channels, clock divider 1
  ADC12CTL1 = ADC12SHP | ADC12SSEL_3 | ADC12CONSEQ_1 | ADC12DIV_1;
  //Input channel select A0, V(R+) = VeREF+ (external Ref)
  ADC12MCTL0 = ADC12INCH_0 | ADC12SREF_2;
  //Input channel select A1, V(R+) = VeREF+ (external Ref), = End of sequence
  ADC12MCTL1 = ADC12INCH_1 | ADC12SREF_2 | ADC12EOS;
  //Enable ADC
  ADC12CTL0 |= ADC12ENC;
}

void GetAudio() {
  int i;
  Ltmpvalue = 0;
  Rtmpvalue = 0;
  for (i = 0; i < 100;i++) {
    ADC12CTL0 |= ADC12SC;
    while (!(ADC12IFG & BIT0));
    while (!(ADC12IFG & BIT1));
    Lvalue=ADC12MEM0;
    Rvalue=ADC12MEM1;
    if ((float)Lvalue > Ltmpvalue) Ltmpvalue = (float)Lvalue;
    if ((float)Rvalue > Rtmpvalue) Rtmpvalue = (float)Rvalue;
  }
}

void fillAudio(uint8_t fLeft, uint8_t fRight) {
    int index;
    for (index=0; index < NUM_LEDS; index++) {
        if (fLeft > index) {
            if (index<=11) strip.setLEDColor((MAX_CH-index-1),fLeft/2, 0x00, fLeft/2);
            else
            if (index<=29) strip.setLEDColor((MAX_CH-index-1),0x00, 0x00, fLeft/2);
            else
            if (index<=47) strip.setLEDColor((MAX_CH-index-1),0x00, fLeft/2, 0x00);
            else strip.setLEDColor((MAX_CH-index-1),fLeft/2, 0x00, 0x00);
        } else strip.setLEDColor((MAX_CH-index-1),0x00, 0x00, 0x00);
        if (fRight >index) {
            if (index<=11) strip.setLEDColor((MAX_CH+index),fRight/2, 0x00, fRight/2);
            else
            if (index<=29) strip.setLEDColor((MAX_CH+index),0x00, 0x00, fRight/2);
            else
            if (index<=47) strip.setLEDColor((MAX_CH+index),0x00, fRight/2, 0x00);
            else strip.setLEDColor((MAX_CH+index),fRight/2, 0x00, 0x00);
        }else strip.setLEDColor((MAX_CH+index),0x00, 0x00, 0x00);
    }
}

void setup()
{
    pinMode(LED_BUILTIN, OUTPUT);
    strip.begin();
    ADCinit();
}

void loop()
{
  if (millis() - timing > 20) {
    timing = millis();
    GetAudio();

    Ltmpvalue=map(Ltmpvalue,LOW_PASS,4096,0,500);
    Rtmpvalue=map(Rtmpvalue,LOW_PASS,4096,0,500);

    Ltmpvalue=constrain(Ltmpvalue,0,500);
    Rtmpvalue=constrain(Rtmpvalue,0,500);

    Ltmpvalue=powf(Ltmpvalue, EXP);
    Rtmpvalue=powf(Rtmpvalue, EXP);

    Lfvalue=Ltmpvalue*SMOOTH + Lfvalue*(1-SMOOTH);
    Rfvalue=Rtmpvalue*SMOOTH + Rfvalue*(1-SMOOTH);

    Llenght = map(Lfvalue, 0, MAXLEVEL, 0, MAX_CH);
    Rlenght = map(Rfvalue, 0, MAXLEVEL, 0, MAX_CH);
    Llenght = constrain(Llenght, 0, MAX_CH);
    Rlenght = constrain(Rlenght, 0, MAX_CH);
    fillAudio(Llenght, Rlenght);
    strip.show();
  }
}
